# from BenchmarkTest import RuntimeBenchmark
# rt = RuntimeBenchmark()
# rt.PyTest()
# rt.LibTest()

import os
import sys
import datetime
import math
import random

import PyQt5.QtCore as QtCore
import sip

win = (sys.platform == "win32")
key = "runtimelite,1000,rud9116709352,none,JFB3LNBHPBK2F5KHT112"

if win:
    os.environ["PATH"] += ";C:\\Program Files (x86)\\ArcGIS SDKs\\Qt100.2.1\\sdk\\windows\\x64\\bin\\release"
    #os.environ["PATH"] += ";C:\\Qt\\Qt5.10.1\\5.10.1\\msvc2017_64\\bin"
else:
    pass

import Runtime.Example
Rt = Runtime.Example.Esri.ArcGISRuntime
import Runtime.DemoLib as RtDemo

class RuntimeBenchmark(QtCore.QObject):
    def __init__(self):
        QtCore.QObject.__init__(self)
        self._bInit = False
        self._app = None
        self._tc = None
        random.seed()

    def Setup(self, app):
        if self._bInit:
            return True
        print("Initializing...")
        self._app = app
        env = Rt.ArcGISRuntimeEnvironment.instance()
        result = env.setLicense(key)
        status = result.licenseStatus()
        if status != Rt.LicenseStatus.Valid:
            print("Invalid license")
            return False
        if not env.initialize():
            print("Initialization failed")
            return False
        print("Initialized.")
        return True

    def Reset(self):
        if self._tc is not None:
            del self._tc
            self._tc = None
        del self._app
        self._app = None
        self._bInit = False

    def PyTest(self):
        app = QtCore.QCoreApplication(sys.argv)
        if not self.Setup(app):
            self.Reset()
            return
        print("Running Python benchmark...")
        iNumPolygons = 100000
        iRec = 0
        dSumArea = 0.0
        iSumVertices = 0

        # Begin benchmark
        self.Status("Processing...", False)
        dtStart = datetime.datetime.now()
        for i in range(iNumPolygons):

            try:

                # Create random polygon
                result = self.GeneratePolygon()
                if result[0] == 0:
                    continue # This should never happen
                iSumVertices += result[0]
                dSumArea += result[1]
                
            except Exception as e:
                self.Status(str(e), True)
                return
                
            iRec += 1
            if iRec % 10000 == 0:
                self.Status("Processing..." + str(iRec), False)

        # Completed
        dArea = 0.0
        dNumVertices = 0.0
        if iRec > 0:
            dArea = dSumArea / iRec
            dNumVertices = iSumVertices / iRec
        sMsg = "Avg area = " + format(dArea, ".0f")
        sMsg += " / Avd pts = " + format(dNumVertices, ".0f")
        self.Status(sMsg, False)
        dtEnd = datetime.datetime.now()
        dtDiff = dtEnd - dtStart
        iElapsed = math.floor(dtDiff.total_seconds() + 0.5)
        self.Completed(iElapsed)
        self.Reset()

    def LibTest(self):
        app = QtCore.QCoreApplication(sys.argv)
        if not self.Setup(app):
            self.Reset()
            return
        print("Running DemoLib benchmark...")
        tc = RtDemo.TestClass()
        self._tc = tc
        tc.message.connect(self.Status)
        tc.benchmarkCompleted.connect(self.Completed)
        tc.runBenchmark()
        app.exec_()
        self.Reset()

    def Status(self, msg, error):
        if error:
            print("ERROR: " + msg)
        else:
            print(msg)

    def Completed(self, elapsed):
        print("Seconds elapsed: " + str(elapsed))
        self._app.quit()

    def GeneratePolygon(self):

        # Set up random point generation
        iNumPoints = 100
        sr = Rt.SpatialReference.webMercator()
        dXMin = -12432342
        dYMin = 4182468
        dXMax = -12432184
        dYMax = 4182624

        # Generate random Multipoint
        pc = Rt.PointCollection(sr)
        for i in range(100):
            dX = random.uniform(dXMin, dXMax)
            dY = random.uniform(dYMin, dYMax)
            pc.addPoint(dX, dY)
        mb = Rt.MultipointBuilder(sr)
        mb.setPoints(pc)
        mp = mb.toMultipoint()

        # Generate convex hull
        geom = Rt.GeometryEngine.convexHull(mp)
        if (geom.isEmpty()):
            return [0, 0] # This should never happen
        poly = sip.cast(geom, Rt.Polygon)
        ipc = poly.parts()
        ip = ipc.part(0)

        # Get area and vertex count
        result = []
        result.append(ip.pointCount())
        result.append(Rt.GeometryEngine.area(poly))
        return result

