import os
import sys

initialWidth = 800
initialHeight = 600

win = (sys.platform == "win32")
key = "runtimelite,1000,rud9116709352,none,JFB3LNBHPBK2F5KHT112"

if win:
    os.environ["PATH"] += ";C:\\Program Files (x86)\\ArcGIS SDKs\\Qt100.2.1\\sdk\\windows\\x64\\bin\\release"
    #os.environ["PATH"] += ";C:\\Qt\\Qt5.10.1\\5.10.1\\msvc2017_64\\bin"

import PyQt5.QtCore as QtCore
import PyQt5.QtWidgets as QtWidgets

import Runtime.Example
Rt = Runtime.Example.Esri.ArcGISRuntime

class RuntimeWidgets(QtWidgets.QMainWindow):
    def __init__(self, parent=None):
        super(RuntimeWidgets, self).__init__(parent)
        self._bInit = False
        self._sb = None
        self._mv = None

    def Setup(self):
        if self._bInit:
            return True

        # Initialize Runtime
        print("Initializing...")
        env = Rt.ArcGISRuntimeEnvironment.instance()
        result = env.setLicense(key)
        status = result.licenseStatus()
        if status != Rt.LicenseStatus.Valid:
            print("Invalid license")
            return False
        if not env.initialize():
            print("Initialization failed")
            return False
        lic = env.license()
        ll = lic.licenseLevel()
        msg = "Initialized at license level: "
        if ll == Rt.LicenseLevel.Developer:
            msg += "Developer"
        elif ll == Rt.LicenseLevel.Lite:
            msg += "Lite"
        elif ll == Rt.LicenseLevel.Basic:
            msg += "Basic"
        elif ll == Rt.LicenseLevel.Standard:
            msg += "Standard"
        elif ll == Rt.LicenseLevel.Advanced:
            msg += "Advanced"
        print(msg)

        # Set up window
        self.setWindowTitle("QtWidgets")
        self.resize(initialWidth, initialHeight)

        # Add status bar
        sb = self.statusBar()
        self._sb = sb
        
        # Add MapView
        mv = Rt.MapGraphicsView(self)
        self._mv = mv
        mv.mapScaleChanged.connect(self.ScaleChanged)
        mv.setWrapAroundMode(Rt.WrapAroundMode.Disabled)
        bm = Rt.Basemap.topographic(self)
        mp = Rt.Map(bm, self)
        mv.setMap(mp)
        self.setCentralWidget(mv)

        self._bInit = True
        return True
        print("Ready!")

    @QtCore.pyqtSlot()
    def ScaleChanged(self):
        d = self._mv.mapScale()
        msg = "Scale: " + format(d, ".0f")
        self._sb.showMessage(msg)
        
app = QtWidgets.QApplication(sys.argv)
if win:
    QtCore.QCoreApplication.setAttribute(QtCore.Qt.AA_UseOpenGLES)
rt = RuntimeWidgets()
if rt.Setup():
    rt.show()
else:
    print("Error initializing")

if __name__ == "__main__":
    app.exec_()
    del rt
    del app

