import os
import sys

import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtQml as QtQml
import PyQt5.QtQuick as QtQuick
import PyQt5.QtWidgets as QtWidgets
import sip

qml = "ShowLegend.qml"
win = (sys.platform == "win32")

if win:
    os.environ["PATH"] += ";C:\\Program Files (x86)\\ArcGIS SDKs\\Qt100.2.1\\sdk\\windows\\x64\\bin\\release"
    #os.environ["PATH"] += ";C:\\Qt\\Qt5.10.1\\5.10.1\\msvc2017_64\\bin"

import Runtime.Qt
Rt = Runtime.Qt.Esri.ArcGISRuntime

class ShowLegend(QtCore.QObject):
    def __init__(self):
        QtCore.QObject.__init__(self)
        self._bInit = False
        self._app = None
        self._engine = None
        self._qw = None
        self._mc = None
        self._mv = None
        self._lv = None
        self._li = None

    def Setup(self):
        if self._bInit:
            return True

        # Load QML and get window
        self._app = QtWidgets.QApplication(sys.argv)
        if win:
            QtCore.QCoreApplication.setAttribute(QtCore.Qt.AA_UseOpenGLES)
        engine = QtQml.QQmlApplicationEngine()
        if win:
            engine.addImportPath("C:\\Program Files (x86)\\ArcGIS SDKs\\Qt100.2.1\\sdk\\windows\\x64\\qml")
            engine.addImportPath("C:\\Program Files (x86)\\ArcGIS SDKs\\Qt100.2.1\\sdk\\toolkit\\Import")
        else:
            home = os.environ["HOME"]
            engine.addImportPath(home + "/ArcGIS_SDKs/Qt100.2.1/sdk/macOS/x64/qml")
            engine.addImportPath(home + "/ArcGIS_SDKs/Qt100.2.1/sdk/toolkit/Import")
        self._engine = engine
        engine.load(QtCore.QUrl("../Qt/RuntimeQML/" + qml))
        rootlist = engine.rootObjects()
        if len(rootlist) == 0:
            print("Error loading QML.")
            return False
        qw = rootlist[0]
        self._qw = qw

        # Get UI items
        mc = qw.findChild(QtQuick.QQuickItem, "mapViewContainer")
        self._mc = mc
        lv = qw.findChild(QtQuick.QQuickItem, "legendListView")
        self._lv = lv

        # Add MapView
        bQmlTest = False
        bPropertyTest = True
        mv = Rt.MapQuickView(mc)
        self._mv = mv

        # Add Map
        bm = Rt.Basemap.topographic(self._mv)
        map = Rt.Map(bm, self._mv)
        map.setAutoFetchLegendInfos(True)
        vp = Rt.Viewpoint(Rt.Point(-11e6, 6e6, Rt.SpatialReference(3857)), 9e7)
        map.setInitialViewpoint(vp)
        self._mv.setMap(map)

        # Add layers
        op = map.operationalLayers()
        tl = Rt.ArcGISTiledLayer(QtCore.QUrl("http://services.arcgisonline.com/ArcGIS/rest/services/Specialty/Soil_Survey_Map/MapServer"), mc)
        op.append(tl)
        ml = Rt.ArcGISMapImageLayer(QtCore.QUrl("http://sampleserver6.arcgisonline.com/arcgis/rest/services/Census/MapServer"), mc)
        op.append(ml)
        ft = Rt.ServiceFeatureTable(QtCore.QUrl("http://sampleserver6.arcgisonline.com/arcgis/rest/services/Recreation/FeatureServer/0"), mc)
        fl = Rt.FeatureLayer(ft, mc)
        op.append(fl)

        # Connect signals
        mc.widthChanged.connect(self.SizeChanged)
        mc.heightChanged.connect(self.SizeChanged)
        li = map.legendInfos()
        li.fetchLegendInfosCompleted.connect(self.SetLegendModel)
        self._li = li
        self._bInit = True
        return True

    def Show(self):
        if not self._bInit:
            return
        self._qw.show()

    def Exec(self):
        self._app.exec_()

    @QtCore.pyqtSlot()
    def SizeChanged(self):
        self._mv.setWidth(self._mc.width())
        self._mv.setHeight(self._mc.height())

    @QtCore.pyqtSlot()
    def SetLegendModel(self):
        self._lv.setProperty("model", self._li)

if __name__ == "__main__":
    rt = ShowLegend()
    if rt.Setup():
        rt.Show()
        rt.Exec()
        del rt
    print("--end--")

