// Copyright 2016 ESRI
//
// All rights reserved under the copyright laws of the United States
// and applicable international laws, treaties, and conventions.
//
// You may freely redistribute and use this sample code, with or
// without modification, provided you include the original copyright
// notice and use restrictions.
//
// See the Sample code usage restrictions document for further information.
//

#include <QMessageBox>

#include <testclass.h>

#include "DemoLibTest.h"

DemoLibTest::DemoLibTest(QQuickItem* parent /* = nullptr */): QQuickItem(parent)
{
    _bBusy = false;
    _lblStatus = nullptr;
    _tc = nullptr;
}

DemoLibTest::~DemoLibTest()
{
    if (_tc)
        _tc->deleteLater();
}

void DemoLibTest::componentComplete()
{
    QQuickItem::componentComplete();

    // find QML Label components
    _lblStatus = findChild<QQuickItem*>("lblStatus");
    _lblComplete = findChild<QQuickItem*>("lblComplete");

    // Create TestClass
    _tc = new TestClass;
    connect(_tc, &TestClass::message, this, &DemoLibTest::handleMessage);
    connect(_tc, &TestClass::benchmarkCompleted, this, &DemoLibTest::handleCompleted);

}

void DemoLibTest::runTest()
{
    if (_bBusy)
    {
        this->status(QString("Test already running"), true);
        return;
    }

    _bBusy = true;
    _lblComplete->setProperty("text", QString(""));
    _tc->runBenchmark();
}

void DemoLibTest::handleMessage(QString message, bool error)
{
    this->status(message, error);
}

void DemoLibTest::handleCompleted(qint64 secondsElapsed)
{
    QString msg = QString("Seconds elapsed: ") + QString::number(secondsElapsed);
    _lblComplete->setProperty("text", msg);
    _bBusy = false;
}

void DemoLibTest::status(QString msg, bool error)
{
    if (error)
        _lblStatus->setProperty("color", "red");
    else
        _lblStatus->setProperty("color", "black");
    _lblStatus->setProperty("text", msg);
}
