# from build import Builder, BuildOption
# b = Builder()
# b.Build("Qt", BuildOption.SIP)

import glob
import os
import os.path
import shutil
import subprocess
import sys
from enum import Enum

win = (sys.platform == "win32")

sPath = os.environ["PATH"]
if not "Qt5.10.1" in sPath:
    if win:
        os.environ["PATH"] += ";C:\\Qt\\Qt5.10.1\\5.10.1\\msvc2017_64\\bin"
    else:
        sHome = os.environ["HOME"]
        os.environ["PATH"] += ":" + sHome + "/Qt5.10.1/5.10.1/clang_64/bin"

class BuildOption(Enum):
    SIP = 1
    QMAKE = 2
    MAKE = 3
    INSTALL = 4

class Builder(object):

    def Clean(self, sModuleName, bCleanTarget = False):
        if win:
            sTargetSuffix = ".pyd"
            sHome = os.environ["USERPROFILE"]
            sTargetPath = sHome + "\\AppData\\Local\\Programs\\Python\\Python36\\Lib\\site-packages\\Runtime\\"
            sModPath = ".\\" + sModuleName + "\\"
        else:
            sTargetSuffix = ".so"
            sHome = os.environ["HOME"]
            sTargetPath = "/Library/Frameworks/Python.framework/Versions/3.6/lib/python3.6/site-packages/Runtime/"
            sTargetFile = sTargetPath + "/" + sModuleName + sTargetSuffix
            sModPath = "./" + sModuleName + "/"
        if not os.path.exists(sModPath):
            print("Module folder not found.")
            return False

        # Clean target folder
        if not os.path.exists(sTargetPath):
            os.mkdir(sTargetPath)
        if bCleanTarget:
            sTargetFile = sTargetPath + sModuleName + sTargetSuffix
            if os.path.exists(sTargetFile):
                os.remove(sTargetFile)

        # Clean module folder
        sClean = sModPath + sModuleName + ".so"
        if os.path.exists(sClean):
            os.remove(sClean)
        sClean = sModPath + sModuleName + ".pyd"
        if os.path.exists(sClean):
            os.remove(sClean)
        sClean = sModPath + "lib" + sModuleName + ".dylib"
        if os.path.exists(sClean):
            os.remove(sClean)
        # [TO DO: check if anything else starting with "." is created in macOS]
        sClean = sModPath + ".qmake.stash"
        if os.path.exists(sClean):
            os.remove(sClean)
        sClean = sModPath + "debug"
        if os.path.exists(sClean):
            shutil.rmtree(sClean)
        sClean = sModPath + "release"
        if os.path.exists(sClean):
            shutil.rmtree(sClean)
        for sFile in glob.glob(sModPath + "Makefile*"):
            os.remove(sFile)
        for sFile in glob.glob(sModPath + "sip*.*"):
            os.remove(sFile)
        return True

    def Build(self, sModuleName, option = BuildOption.INSTALL):
        print("Building " + sModuleName + "...")
        print("   Cleaning...")
        if option == BuildOption.INSTALL:
            if not self.Clean(sModuleName, True):
                return
        os.chdir(sModuleName)
        
        # sip
        print("   Running sip...")
        sSipFile = sModuleName + ".sip"
        if win:
            args = ["sip", "-c", ".", 
                    "-I", "../sip", 
                    "-I", "C:\\Program Files (x86)\\ArcGIS SDKs\\Qt100.2.1\\sdk\\include", 
                    "-t", "WS_WIN", "-t", "Qt_5_10_1", sSipFile]
        else:
            args = ["sip", "-c", ".", 
                    "-I", "../sip", 
                    "-I", "~/ArcGIS_SDKs/Qt100.2.1/sdk/include",
                    "-t", "WS_MACX", "-t", "Qt_5_10_1", sSipFile]
        result = subprocess.run(args)
        if result.returncode != 0:
            print("Error running sip")
            os.chdir("..")
            return
        if option == BuildOption.SIP:
            os.chdir("..")
            return

        # qmake
        print("   Running qmake...")
        sProFile = sModuleName + ".pro"
        args = ["qmake", "-o", "Makefile", sProFile]
        result = subprocess.run(args)
        if result.returncode != 0:
            print("Error running qmake")
            os.chdir("..")
            return
        if option == BuildOption.QMAKE:
            os.chdir("..")
            return

        # make
        print("   Running make...")
        if win:
            args = ["nmake", "/f", "Makefile"]
        else:
            args = ["make"]
        result = subprocess.run(args)
        if result.returncode != 0:
            print("Error running make")
            os.chdir("..")
            return
        if option == BuildOption.MAKE:
            os.chdir("..")
            return

        # make install
        print("   Running make install...")
        if win:
            args = ["nmake", "install"]
        else:
            args = ["make", "install"]
        result = subprocess.run(args)
        if result.returncode != 0:
            print("Error running make install")
            os.chdir("..")
            return

        os.chdir("..")
        print("   Cleaning up...")
        self.Clean(sModuleName, False)
        return

